@@ -673,7 +673,7 @@ svyvar.survey.design<-function(x, design, na.rm=FALSE,...){
 	else if(typeof(x) %in% c("expression","symbol"))
             x<-eval(x, design$variables)
         
-	n<-NROW(x)
+	n<-sum(weights(design,"sampling")!=0)
 	xbar<-svymean(x,design, na.rm=na.rm)
 	if(NCOL(x)==1) {
             x<-x-xbar
